# -*- coding: utf-8 -*-

'''
Expectations that can set on a stub.
'''

import inspect
from comparators import *
from exception import *
from termcolor import colored


class ExpectationRule(object):
  def __init__(self, *args, **kwargs):
    self._passed = False

  def validate(self, *args, **kwargs):
    raise NotImplementedError("Must be implemented by subclasses")
  
class ArgumentsExpectationRule(ExpectationRule):
  def __init__(self, *args, **kwargs):
    super(ArgumentsExpectationRule, self).__init__(*args, **kwargs)
    self.set_args( *args, **kwargs )
  
  def set_args(self, *args, **kwargs):
    self.args = []
    self.kwargs = {}
    
    # Convert all of the arguments to comparators
    self.args = build_comparators(*args)
    self.kwargs = dict([(k, build_comparators(v)[0]) for k, v in kwargs.iteritems()])

  def validate(self, *args, **kwargs):
    self.in_args = args[:]
    self.in_kwargs = kwargs.copy()

    # First just check that the number of arguments is the same or different
    if len(args)!=len(self.args) or len(kwargs)!=len(self.kwargs):
      self._passed = False
      return False

    for x in xrange(len(self.args)):
      if not self.args[x].test( args[x] ):
        self._passed = False
        return False

    for arg_name,arg_test in self.kwargs.iteritems():
      try:
        value=kwargs.pop(arg_name)
      except KeyError:
        self._passed = False
        return False
      if not arg_test.test( value ):
        self._passed = False
        return False

    # If there are arguments left over, is error
    if len(kwargs):
      self._passed = False
      return False

    self._passed = True
    return self._passed
  
  @classmethod
  def pretty_format_args(self, *args, **kwargs):
    """
    Take the args, and kwargs that are passed them and format in a prototype style.
    """
    args = list([repr(a) for a in args])
    for key, value in kwargs.iteritems():
      args.append("%s=%s" % (key, repr(value)))
    
    return "(%s)" % ", ".join([a for a in args])

  def __str__(self):
    if hasattr(self, 'in_args') and hasattr(self, 'in_kwargs'):
      return "\tExpected: %s\n\t\t    Used: %s" % \
        (self.pretty_format_args(*self.args, **self.kwargs), self.pretty_format_args(*self.in_args, **self.in_kwargs))
        
    return "\tExpected: %s" % \
        (self.pretty_format_args(*self.args, **self.kwargs))

class Expectation(object):
  '''
  Encapsulate an expectation.
  '''

  def __init__(self, stub):
    self._met = False
    self._stub = stub
    self._arguments_rule = ArgumentsExpectationRule()
    self._raises = None
    self._returns = None
    self._max_count = self._min_count = 1
    self._run_count = 0 
    self._any_order = False
    self._side_effect = False
    self._side_effect_args = None
    self._side_effect_kwargs = None
    self._teardown = False
    self._any_args = False

  # Support expectations as context managers. See
  #   https://github.com/agoragames/chai/issues/1
  def __enter__(self):
    return self._returns

  def __exit__(*args):
    pass
    
  def args(self, *args, **kwargs):
    """
    Creates a ArgumentsExpectationRule and adds it to the expectation
    """
    self._any_args = False
    self._arguments_rule.set_args(*args, **kwargs)
    return self

  def any_args(self):
    '''
    Accept any arguments passed to this call.
    '''
    self._any_args = True
    return self

  def returns(self, value):
    """
    What this expectation should return
    """
    self._returns = value
    return self

  def raises(self, exception):
    """
    Adds a raises to the expectation, this will be raised when the expectation is met.
    
    This can be either the exception class or instance of a exception
    """
    self._raises = exception
    return self

  def times(self, count):
    self._min_count = self._max_count = count
    return self
  
  def at_least(self, min_count):
    self._min_count = min_count
    self._max_count = None
    return self
  
  def at_least_once(self):
    self.at_least(1)
    return self
  
  def at_most(self, max_count):
    self._max_count = max_count
    return self
  
  def at_most_once(self):
    self.at_most(1)
    return self
  
  def once(self):
    self._min_count = 1
    self._max_count = 1
    return self

  def any_order(self):
    self._any_order = True
    return self

  def side_effect(self, func, *args, **kwargs):
    self._side_effect = func
    self._side_effect_args = args
    self._side_effect_kwargs = kwargs
    return self

  def teardown(self):
    self._teardown = True
    return self
  
  def return_value(self):
    """
    Returns the value for this expectation or raises the proper exception.
    """
    if self._raises:
      # Handle exceptions
      if inspect.isclass(self._raises):
        raise self._raises()
      else:
        raise self._raises
    else:
      if isinstance(self._returns, tuple):
        return tuple([x.value if isinstance(x,Variable) else x for x in self._returns])
      return self._returns.value if isinstance(self._returns,Variable) else self._returns

  def close(self, *args, **kwargs):
    '''
    Mark this expectation as closed. It will no longer be used for matches.
    '''
    # If any_order, then this effectively is never closed. The Stub.__call__ 
    # will just bypass it when it doesn't match. If there is a strict count
    # it will also be bypassed, but if there's just a min set up, then it'll
    # effectively stay open and catch any matching call no matter the order
    if not self._any_order:
      self._met = True
    
  def closed(self, with_counts=False):
    rval = self._met
    if with_counts:
      rval = rval or self.counts_met()
    return rval
  
  def counts_met(self):
    return self._run_count >= self._min_count and not (self._max_count and not self._max_count == self._run_count)
  
  def match(self, *args, **kwargs):
    """
    Check the if these args match this expectation.
    """
    return self._any_args or self._arguments_rule.validate(*args, **kwargs)

  def test(self, *args, **kwargs):
    """
    Validate all the rules with in this expectation to see if this expectation has been met.
    """
    if not self._met:
      if self.match(*args, **kwargs):
        self._run_count += 1
        if not self._max_count == None and self._run_count == self._max_count:
          self._met = True
        if self._side_effect:
          if self._side_effect_args or self._side_effect_kwargs:
            self._side_effect(*self._side_effect_args, **self._side_effect_kwargs)
          else:
            self._side_effect(*args, **kwargs)
      else:
        self._met = False

      # If this is met and we're supposed to tear down, must do it now so that 
      # this stub is not called again
      if self._met and self._teardown:
        self._stub.teardown()

    return self.return_value()
  
  def __str__(self):
    runs_string = "     Ran: %s, Min Runs: %s, Max Runs: %s" % (self._run_count, self._min_count, "∞" if self._max_count == None else self._max_count)
    return_string = "  Raises: %s" % self._raises if self._raises else " Returns: %s" % repr(self._returns)
    return "\n\t%s\n\t%s\n\t\t%s\n\t\t%s" % (colored("%s - %s" % (self._stub.name, "Passed" if self._arguments_rule._passed else "Failed")
      , "green" if self._arguments_rule._passed else "red"), self._arguments_rule, return_string, runs_string)



'''
Tests for the sample module
'''

import os
import random

from chai import Chai
from chai.stub import stub
from chai.exception import *
import samples
from samples import SampleBase, SampleChild

import unittest2 as unittest

class CustomException(Exception): pass

class SampleModuleTest(Chai):

  def test_mod_func_2_as_obj_name(self):
    expect(samples, 'mod_func_1').args(42, foo='bar')
    samples.mod_func_2(42, foo='bar')
  
  def test_mod_func_2_as_obj_ref(self):
    expect(samples.mod_func_1).args(42, foo='bar')
    samples.mod_func_2(42, foo='bar')

class SampleBaseTest(Chai):

  def test_expects_property(self):
    obj = SampleBase()
    expect(obj, 'prop').returns("property value")
    assert_equals("property value", obj.prop)

  def test_expects_on_builtin_function(self):
    # NOTE: os module is a good example where it binds from another
    # (e.g. posix), so it has to use the named reference or else it
    # stubs the original module
    expect( os, 'remove' ).args('foo').returns('ok')
    assert_equals( 'ok', os.remove('foo') )

  def test_expects_bound_method_returns(self):
    obj = SampleBase()
    expect(obj.bound_method).args(1, 2).returns(12)
    assert_equals(12, obj.bound_method(1, 2))

    expect(obj.bound_method).args(1, 4).returns(1100)
    assert_equals(1100, obj.bound_method(1, 4))

  def test_expects_bound_method_at_least_with_other_expectation_and_no_anyorder(self):
    obj = SampleBase()
    expect(obj.bound_method).args(1, 2).returns(12).at_least(2)
    assert_equals(12, obj.bound_method(1, 2))
    assert_equals(12, obj.bound_method(1, 2))

    expect(obj.bound_method).args(1, 3).returns(100)
    assert_equals(100, obj.bound_method(1, 3))
    
    assert_raises(UnexpectedCall, obj.bound_method, 1, 2)

  def test_expects_bound_method_at_least_with_other_expectation_and_anyorder(self):
    obj = SampleBase()
    expect(obj.bound_method).args(1, 2).returns(12).at_least(2).any_order()
    assert_equals(12, obj.bound_method(1, 2))
    assert_equals(12, obj.bound_method(1, 2))

    expect(obj.bound_method).args(1, 3).returns(100)
    assert_equals(100, obj.bound_method(1, 3))
    
    assert_equals(12, obj.bound_method(1, 2))

  def test_expects_bound_method_at_least_as_last_expectation(self):
    obj = SampleBase()
    expect(obj.bound_method).args(1, 2).returns(12).at_least(3)
    assert_equals(12, obj.bound_method(1, 2))
    assert_equals(12, obj.bound_method(1, 2))
    assert_equals(12, obj.bound_method(1, 2))
    assert_equals(12, obj.bound_method(1, 2))

  def test_expects_bound_method_at_most(self):
    obj = SampleBase()
    expect(obj.bound_method).args(1, 2).returns(12).at_most(3)
    assert_equals(12, obj.bound_method(1, 2))
    assert_equals(12, obj.bound_method(1, 2))
    obj.bound_method(1, 2)
    assert_raises(UnexpectedCall, obj.bound_method, 1, 2)

  def tests_expects_bound_method_any_order_with_fixed_maxes(self):
    obj = SampleBase()
    expect(obj.bound_method).args(1).returns(2).any_order()
    expect(obj.bound_method).args(3).returns(4).any_order()
    assert_equals(4, obj.bound_method(3) )
    assert_equals(2, obj.bound_method(1) )
    assert_raises(UnexpectedCall, obj.bound_method, 1)

  def tests_expects_bound_method_any_order_with_mins(self):
    obj = SampleBase()
    expect(obj.bound_method).args(1).returns(2).any_order().at_least_once()
    expect(obj.bound_method).args(3).returns(4).any_order().at_least_once()
    assert_equals(4, obj.bound_method(3) )
    assert_equals(2, obj.bound_method(1) )
    assert_equals(4, obj.bound_method(3) )
    assert_equals(2, obj.bound_method(1) )
    assert_equals(2, obj.bound_method(1) )
    assert_equals(4, obj.bound_method(3) )
    assert_equals(2, obj.bound_method(1) )

  def test_expects_any_order_without_count_modifiers(self):
    obj = SampleBase()
    expect(obj.bound_method).args(3).returns(4)
    expect(obj.bound_method).args(1).returns(2).any_order()
    expect(obj.bound_method).args(3).returns(4)
    assert_equals(4, obj.bound_method(3) )
    assert_equals(4, obj.bound_method(3) )
    assert_equals(2, obj.bound_method(1) )

  def test_expects_bound_method_raises(self):
    obj = SampleBase()
    expect(obj.bound_method).args(1, 2).raises(CustomException)
    assert_raises(CustomException, obj.bound_method, 1, 2)

    expect(obj.bound_method).args(1, 2).raises(CustomException())
    assert_raises(CustomException, obj.bound_method, 1, 2)

  def test_expects_bound_method_can_be_used_for_iterative_testing(self):
    obj = SampleBase()
    expect(obj.bound_method).args(1, 2).returns(12)
    assert_equals(12, obj.bound_method(1, 2))
    assert_raises(UnexpectedCall, obj.bound_method)

    expect(obj.bound_method).args(1, 4).returns(1100)
    assert_equals(1100, obj.bound_method(1, 4))

  def test_stub_bound_method_raises_unexpectedcall(self):
    obj = SampleBase()
    stub(obj.bound_method)
    assert_raises(UnexpectedCall, obj.bound_method)
  
  def test_expect_bound_method_with_equals_comparator(self):
    obj = SampleBase()
    expect(obj.bound_method).args( equals(42) )
    obj.bound_method( 42 )
    assert_raises(UnexpectedCall, obj.bound_method, 32 )

  def test_expect_bound_method_with_is_a_comparator(self):
    obj = SampleBase()
    expect(obj.bound_method).args( is_a(int) )
    obj.bound_method( 42 )
    assert_raises(UnexpectedCall, obj.bound_method, '42' )
  
  def test_expect_bound_method_with_anyof_comparator(self):
    obj = SampleBase()
    expect(obj.bound_method).times(4).args( 
      any_of(int,3.14,'hello',is_a(list)) )
    obj.bound_method( 42 )
    obj.bound_method( 3.14 )
    obj.bound_method( 'hello' )
    obj.bound_method( [1,2,3] )
    assert_raises(UnexpectedCall, obj.bound_method, '42' )

  def test_expect_bound_method_with_allof_comparator(self):
    obj = SampleBase()
    expect(obj.bound_method).args( all_of(bytearray,'hello') )
    obj.bound_method( bytearray('hello') )
    assert_raises(UnexpectedCall, obj.bound_method, 'hello' )
  
  def test_expect_bound_method_with_notof_comparator(self):
    obj = SampleBase()
    expect(obj.bound_method).args( not_of(any_of(float,int)) )
    obj.bound_method( 'hello' )

  def test_expect_bound_method_with_notof_comparator_using_types(self):
    obj = SampleBase()
    expect(obj.bound_method).args( not_of(float,int) )
    obj.bound_method( 'hello' )
    
  def test_expect_unbound_method_acts_as_any_instance(self):
    expect( SampleBase.bound_method ).args('hello').returns('world')
    expect( SampleBase.bound_method ).args('hello').returns('mars')

    obj1 = SampleBase()
    obj2 = SampleBase()
    assert_equals( 'world', obj2.bound_method('hello') )
    assert_equals( 'mars', obj1.bound_method('hello') )
    assert_raises(UnexpectedCall, obj2.bound_method)

  def test_stub_unbound_method_acts_as_no_instance(self):
    stub( SampleBase.bound_method )

    obj1 = SampleBase()
    obj2 = SampleBase()
    assert_raises(UnexpectedCall, obj2.bound_method)
    assert_raises(UnexpectedCall, obj1.bound_method)
  
  def test_expects_class_method(self):
    expect(SampleBase.a_classmethod).returns(12)
    assert_equals(12, SampleBase.a_classmethod())

    obj = SampleBase()
    expect(SampleBase.a_classmethod).returns(100)
    assert_equals(100, obj.a_classmethod())

  def test_stub_class_method(self):
    stub(SampleBase.a_classmethod)
    assert_raises(UnexpectedCall, SampleBase.a_classmethod)

    obj = SampleBase()
    assert_raises(UnexpectedCall, obj.a_classmethod)

  def test_expect_callback(self):
    obj = SampleBase()
    expect(obj.callback_target)
    obj.callback_source()

  def test_add_to_list_with_mock_object(self):
    obj = SampleBase()
    obj._deque = mock()
    expect( obj._deque.append ).args('value')
    obj.add_to_list('value')

  def test_add_to_list_with_module_mock_object(self):
    mock( samples, 'deque' )
    deq = mock()
    expect( samples.deque.__call__ ).returns( deq )
    expect( deq.append ).args('value')

    obj = SampleBase()
    obj.add_to_list('value')
  
  def test_regex_comparator(self):
    obj = SampleBase()
    expect(obj.bound_method).args(matches("name$")).returns(100)
    assert_equals(obj.bound_method('first_name'), 100)

  def test_regex_comparator(self):
    obj = SampleBase()
    expect(obj.bound_method).args(matches("name$")).returns(100)
    assert_equals(obj.bound_method('first_name'), 100)

  def test_ignore_arg(self):
    obj = SampleBase()
    expect(obj.bound_method).args(ignore_arg()).returns(100)
    assert_equals(obj.bound_method('first_name'), 100)

  def test_function_comparator(self):
    obj = SampleBase()
    expect(obj.bound_method).args(func(lambda arg: arg > 10)).returns(100)
    assert_equals(obj.bound_method(100), 100)

  def test_in_comparator(self):
    obj = SampleBase()
    expect(obj.bound_method).args(contains('name')).returns(100).at_most(3)
    assert_equals(obj.bound_method(['name', 'age']), 100)
    assert_equals(obj.bound_method({'name' : 'vitaly'}), 100)
    assert_equals(obj.bound_method('lasfs-name-asfsad'), 100)

  def test_almost_equals_comparator(self):
    obj = SampleBase()
    expect(obj.bound_method).args(almost_equals(10.1234, 2)).returns(100)
    assert_equals(obj.bound_method(10.12), 100)

  def test_is_comparator(self):
    obj = SampleBase()
    expect(obj.bound_method).args(is_arg(obj)).returns(100)
    assert_equals(obj.bound_method(obj), 100)

  def test_var_comparator(self):
    obj = SampleBase()
    expect(obj.add_to_list).args( var('value1') )
    expect(obj.add_to_list).args( var('value2') )
    expect(obj.add_to_list).args( var('value3') ).at_least_once()

    obj.add_to_list('v1')
    obj.add_to_list('v2')
    obj.add_to_list('v3')
    obj.add_to_list('v3')
    self.assertRaises( UnexpectedCall, obj.add_to_list, 'v3a')

    assert_equals( 'v1', var('value1').value )
    assert_equals( 'v2', var('value2').value )
    assert_equals( 'v3', var('value3').value )

class SampleChildTest(Chai):

  def test_stub_base_class_expect_child_classmethod(self):
    stub(SampleBase.a_classmethod)
    expect(SampleChild.a_classmethod)

    SampleChild.a_classmethod()



'''
Tests for the mock object
'''

import unittest
import types

from chai.comparators import *
from chai.mock import Mock
from chai.exception import UnexpectedCall
from chai.stub import stub

class MockTest(unittest.TestCase):

 # def test_get_attribute_creates_a_new_method(self):
 #   m = Mock()
 #   self.assertTrue( isinstance(m.foo, types.MethodType) )
  def test_get_attribute_creates_a_mock_method(self):
    m = Mock()
    self.assertTrue( isinstance(m.foo, Mock) )
    self.assertFalse( m.foo is m )

  def test_get_attribute_caches_auto_methods(self):
    m = Mock()
    self.assertTrue( m.foo is m.foo )

  def test_get_attribute_supports_multiple_depths(self):
    m = Mock()
    self.assertTrue( isinstance(m.foo.bar, Mock) )

  def test_get_attribute_auto_method_raises_unexpectedcall_when_unstubbed(self):
    m = Mock()
    self.assertRaises( UnexpectedCall, m.foo )

  def test_get_attribute_auto_method_raises_unexpectedcall_when_stubbed(self):
    m = Mock()
    stub( m.foo )
    self.assertRaises( UnexpectedCall, m.foo )

  def test_get_attribute_auto_method_not_raises_unexpectedcall_when_stubbed(self):
    m = Mock()
    stub(m.foo).expect().returns('success')
    self.assertEquals( 'success', m.foo() )

  def test_get_attribute_auto_method_not_raises_unexpectedcall_multiple_depths(self):
    m = Mock()
    stub(m.foo.bar).expect().returns('success')
    self.assertEquals( 'success', m.foo.bar() )

  def test_get_attribute_does_not_overwrite_existing_attr(self):
    m = Mock()
    m.foo = 42
    self.assertEquals( 42, m.foo )

  def test_call_raises_unexpectedcall_when_unstubbed(self):
    m = Mock()
    self.assertRaises( UnexpectedCall, m )

  def test_call_not_raises_unexpectedcall_when_stubbed(self):
    m = Mock()
    stub(m).expect().returns('success')
    self.assertEquals( 'success', m() )

  def test_nonzero_returns_true_when_unstubbed(self):
    m = Mock()
    self.assertTrue( m.__nonzero__() )

  def test_nonzero_when_stubbed(self):
    m = Mock()
    stub(m.__nonzero__).expect().returns(False)
    self.assertFalse( m.__nonzero__() )

  def test_container_interface_when_unstubbed(self):
    m = Mock()
    self.assertRaises( UnexpectedCall, len, m )
    self.assertRaises( UnexpectedCall, m.__getitem__, 'foo' )
    self.assertRaises( UnexpectedCall, m.__setitem__, 'foo', 'bar' )
    self.assertRaises( UnexpectedCall, m.__delitem__, 'foo' )
    self.assertRaises( UnexpectedCall, iter, m )
    self.assertRaises( UnexpectedCall, reversed, m )
    self.assertRaises( UnexpectedCall, m.__contains__, 'foo' )
    self.assertRaises( UnexpectedCall, m.__getslice__, 'i', 'j')
    self.assertRaises( UnexpectedCall, m.__setslice__, 'i', 'j', 'foo')
    self.assertRaises( UnexpectedCall, m.__delslice__, 'i', 'j')

  def test_container_interface_when_stubbed(self):
    m = Mock()
    i = iter([1,2,3])

    stub( m.__len__ ).expect().returns( 42 )
    stub( m.__getitem__ ).expect().args('foo').returns('getitem')
    stub( m.__setitem__ ).expect().args('foo', 'bar')
    stub( m.__delitem__ ).expect().args('foo')
    stub( m.__iter__ ).expect().returns( i )
    stub( m.__reversed__ ).expect().returns( 'backwards' )
    stub( m.__contains__ ).expect().args('foo').returns( True )

    self.assertEquals( 42, len(m) )
    self.assertEquals( 'getitem', m['foo'] )
    m['foo']='bar'
    del m['foo']
    self.assertEquals( i, iter(m) )
    self.assertEquals( 'backwards', reversed(m) )
    self.assertEquals( True, 'foo' in m )

  def test_context_manager_interface_when_unstubbed(self):
    m = Mock()

    def cm():
      with m:
        pass

    self.assertRaises( UnexpectedCall, cm )

  def test_context_manager_interface_when_stubbed(self):
    m = Mock()
    exc = Exception('fail')

    def cm_noraise():
      with m: pass

    def cm_raise():
      with m:
        raise exc

    stub( m.__enter__ ).expect().times(2)
    stub( m.__exit__ ).expect().args( None, None, None )
    stub( m.__exit__ ).expect().args( Is(Exception), exc, types.TracebackType )

    cm_noraise()
    self.assertRaises( Exception, cm_raise )



'''
The chai test class.
'''

try:
  import unittest2
  unittest = unittest2
except ImportError:
  import unittest

import re
import sys
import inspect

from exception import *
from mock import Mock
from stub import stub
from collections import deque
from comparators import *


class ChaiTestType(type):
  """
  Metaclass used to wrap all test methods to make sure the assert_expectations
  in the correct context.
  """

  def __init__(cls, name, bases, d):
    type.__init__(cls, name, bases, d)

    # also get all the attributes from the base classes to account
    # for a case when test class is not the immediate child of MoxTestBase
    for base in bases:
      for attr_name in dir(base):
        d[attr_name] = getattr(base, attr_name)

    for func_name, func in d.items():
      if func_name.startswith('test') and callable(func):
        setattr(cls, func_name, ChaiTestType.test_wrapper(cls, func))

  @staticmethod
  def test_wrapper(cls, func):
    """
    Wraps a test method, when that test method has completed it 
    calls assert_expectations on the stub. This is to avoid getting to exceptions about the same error.
    """
    def wrapper(self, *args, **kwargs):
      func(self, *args, **kwargs)
      exceptions = []
      for stub in self._stubs:
        exceptions.extend(stub.unmet_expectations())
      
      if exceptions:
        raise ExpectationNotSatisfied(*exceptions)
      
    wrapper.__name__ = func.__name__
    wrapper.__doc__ = func.__doc__
    wrapper.__module__ = func.__module__
    wrapper.__wrapped__ = func
    return wrapper

class Chai(unittest.TestCase):
  '''
  Base class for all tests
  '''
  
  __metaclass__ = ChaiTestType

  # When initializing, alias all the cAmElCaSe methods to more helpful ones
  def __init__(self, *args, **kwargs):
    super(Chai,self).__init__(*args, **kwargs)
    for attr in dir(self):
      if attr.startswith('assert') and attr!='assert_':
        pieces = ['assert'] + re.findall('[A-Z][a-z]+', attr[5:])
        name = '_'.join( [s.lower() for s in pieces] )
        setattr(self, name, getattr(self,attr))

  # Load in the comparators
  equals = Equals
  almost_equals = AlmostEqual
  is_a = IsA
  is_arg = Is
  any_of = Any
  all_of = All
  not_of = Not
  matches = Regex
  func = Function 
  ignore_arg = Ignore
  ignore = Ignore
  in_arg = In
  contains = Contains
  var = Variable
  like = Like

  def setUp(self):
    super(Chai,self).setUp()

    # Setup stub tracking
    self._stubs = deque()

    # Setup mock tracking
    self._mocks = deque()

    # Try to load this into the module that the test case is defined in, so
    # that 'self.' can be removed. This has to be done at the start of the test
    # because we need the reference to be correct at the time of test run, not
    # when the class is defined or an instance is created.
    mod = sys.modules[ self.__class__.__module__ ]
    for attr in dir(self):
      if attr.startswith('assert'):
        setattr(mod, attr, getattr(self, attr) )
      elif isinstance(getattr(self,attr), type) and issubclass( getattr(self,attr), Comparator ):
        setattr(mod, attr, getattr(self, attr) )
    setattr(mod, 'stub', self.stub)
    setattr(mod, 'expect', self.expect)
    setattr(mod, 'mock', self.mock)
    

  # Because cAmElCaSe sucks
  setup = setUp

  def tearDown(self):
    super(Chai,self).tearDown()

    # Docs insist that this will be called no matter what happens in runTest(),
    # so this should be a safe spot to unstub everything
    exception = None
    while len(self._stubs):
      stub = self._stubs.popleft()
      stub.teardown() # Teardown the reset of the stub
    
    # Do the mocks in reverse order in the rare case someone called mock(obj,attr)
    # twice.
    while len(self._mocks):
      mock = self._mocks.pop()
      if len(mock)==2:
        delattr( mock[0], mock[1] )
      else:
        setattr( mock[0], mock[1], mock[2] )

    # Clear out any cached variables
    Variable.clear()
    
    # Lastly, if there were any errors, raise them
    if exception:
      raise exception

  # Because cAmElCaSe sucks
  teardown = tearDown

  def stub(self, obj, attr=None):
    '''
    Stub an object. If attr is not None, will attempt to stub that attribute
    on the object. Only required for modules and other rare cases where we
    can't determine the binding from the object.
    '''
    s = stub(obj, attr)
    if s not in self._stubs:
      self._stubs.append( s )
    return s

  def expect(self, obj, attr=None):
    '''
    Open and return an expectation on an object. Will automatically create a
    stub for the object. See stub documentation for argument information.
    '''
    return self.stub(obj,attr).expect()

  def mock(self, obj=None, attr=None, **kwargs):
    '''
    Return a mock object.
    '''
    rval = Mock(**kwargs)
    if obj!=None and attr!=None:
      rval._object = obj
      rval._attr = attr
      
      if hasattr(obj,attr):
        orig = getattr(obj, attr)
        self._mocks.append( (obj,attr,orig) )
        setattr(obj, attr, rval)
      else:
        self._mocks.append( (obj,attr) )
        setattr(obj, attr, rval)
    return rval



'''
An open mocking object.
'''
from types import MethodType
from stub import stub, Stub
from exception import UnexpectedCall

class Mock(object):
  '''
  A class where all calls are stubbed.
  '''

  def __init__(self, **kwargs):
    for name, value in kwargs.iteritems():
      setattr(self, name, value)

  # For whatever reason, new-style objects require this method defined before
  # any instance is created. Defining it through __getattr__ is not enough. This
  # appears to be a bug/feature in new classes where special members, or at least
  # __call__, have to defined when the instance is created. Also, if it's already
  # defined on the instance, getattr() will return the stub but the original
  # method will always be called. Anyway, it's all crazy, but that's why the 
  # implementation of __call__ is so weird.
  def __call__(self, *args, **kwargs):
    if isinstance(getattr(self,'__call__'), Stub):
      return getattr(self,'__call__')(*args, **kwargs)
    raise UnexpectedCall()

  def __getattr__(self,name):
    rval = self.__dict__.get(name)

    if not rval or not isinstance(rval,(Stub,Mock)):
      rval = Mock()
      rval._name = name
      setattr(self, name, rval)

    return rval

  ###
  ### Define nonzero so that basic "if <mock>:" stanzas will work.
  ###
  def __nonzero__(self):
    if isinstance(getattr(self,'__nonzero__'), Stub):
      return getattr(self,'__nonzero__')()
    return True

  ###
  ### Emulate container types, the 99% of cases where we want to mock the
  ### special methods. They all raise UnexpectedCall unless they're mocked out
  ### http://docs.python.org/reference/datamodel.html#emulating-container-types
  ###
  # HACK: it would be nice to abstract this lookup-stub behavior in a decorator
  # but that gets in the way of stubbing. Would like to figure that out @AW
  def __len__(self):
    if isinstance(getattr(self,'__len__'), Stub):
      return getattr(self,'__len__')()
    raise UnexpectedCall()
    
  def __getitem__(self, key):
    if isinstance(getattr(self,'__getitem__'), Stub):
      return getattr(self,'__getitem__')(key)
    raise UnexpectedCall()

  def __setitem__(self, key, value):
    if isinstance(getattr(self,'__setitem__'), Stub):
      return getattr(self,'__setitem__')(key, value)
    raise UnexpectedCall()

  def __delitem__(self, key):
    if isinstance(getattr(self,'__delitem__'), Stub):
      return getattr(self,'__delitem__')(key)
    raise UnexpectedCall()

  def __iter__(self):
    if isinstance(getattr(self,'__iter__'), Stub):
      return getattr(self,'__iter__')()
    raise UnexpectedCall()

  def __reversed__(self):
    if isinstance(getattr(self,'__reversed__'), Stub):
      return getattr(self,'__reversed__')()
    raise UnexpectedCall()

  def __contains__(self, item):
    if isinstance(getattr(self,'__contains__'), Stub):
      return getattr(self,'__contains__')(item)
    raise UnexpectedCall()

  ###
  ### Emulate context managers
  ### http://docs.python.org/reference/datamodel.html#with-statement-context-managers
  ###
  def __enter__(self):
    if isinstance(getattr(self,'__enter__'), Stub):
      return getattr(self,'__enter__')()
    raise UnexpectedCall()
  
  def __exit__(self, exc_type, exc_value, traceback):
    if isinstance(getattr(self,'__exit__'), Stub):
      return getattr(self,'__exit__')(exc_type, exc_value, traceback)
    raise UnexpectedCall()




import unittest

from chai.comparators import *

class ComparatorsTest(unittest.TestCase):
  
  def test_build_comparators_builds_equals(self):
    comp = build_comparators("str")[0]
    self.assertTrue(isinstance(comp, Equals))
    comp = build_comparators(12)[0]
    self.assertTrue(isinstance(comp, Equals))
    comp = build_comparators(12.1)[0]
    self.assertTrue(isinstance(comp, Equals))
    comp = build_comparators([])[0]
    self.assertTrue(isinstance(comp, Equals))
    comp = build_comparators({})[0]
    self.assertTrue(isinstance(comp, Equals))
    comp = build_comparators(tuple())[0]
    self.assertTrue(isinstance(comp, Equals))
  
  def test_build_comparators_is_a(self):
    class CustomObject(object): pass
    comp = build_comparators(CustomObject)[0]
    self.assertTrue(isinstance(comp, IsA))
    
  def test_build_comparators_passes_comparators(self):
    any_comp = Any()
    comp = build_comparators(any_comp)[0]
    self.assertTrue(comp is any_comp)

  def test_equals(self):
    comp = Equals(3)
    self.assertTrue( comp.test(3) )
    self.assertTrue( comp.test(3.0) )
    self.assertFalse( comp.test('3') )
  
  def test_equals_repr(self):
      comp = Equals(3)
      self.assertEquals(str(comp), "3")

  def test_eq(self):
    comp = Equals(3)
    self.assertEquals( comp, 3 )

  def test_is_a(self):
    comp = IsA(str)
    self.assertTrue( comp.test('foo') )
    self.assertFalse( comp.test(u'foo') )
    self.assertFalse( comp.test(bytearray('foo')) )
    
    comp = IsA((str,int))
    self.assertTrue( comp.test('') )
    self.assertTrue( comp.test(42) )
    self.assertFalse( comp.test(3.14) )

  def test_is_a_repr(self):
    comp = IsA(str)
    self.assertEquals(repr(comp), "IsA(str)")
    
  def test_is_a_format_name(self):
    comp = IsA(str)
    self.assertEquals(comp._format_name(), "str")
    comp = IsA((str, list))
    self.assertEquals(comp._format_name(), "['str', 'list']")

  def test_is(self):
    class Test(object):
      def __eq__(self, other): return True

    obj1 = Test()
    obj2 = Test()
    comp = Is(obj1)
    self.assertEquals( obj1, obj2 )
    self.assertTrue( comp.test(obj1) )
    self.assertFalse( comp.test(obj2) )
  
  def test_is_repr(self):
    class TestObj(object):
      
      def __str__(self):
        return "An Object"
    
    obj = TestObj()
    self.assertEquals(repr(Is(obj)), "Is(An Object)" )

  def test_almost_equal(self):
    comp = AlmostEqual(3.14159265, 3)
    self.assertTrue( comp.test(3.1416) )
    self.assertFalse( comp.test(3.14) )
  
  def test_almost_equal_repr(self):
    comp = AlmostEqual(3.14159265, 3)
    self.assertEquals(repr(comp), "AlmostEqual(value: 3.14159265, places: 3)")
  
  def test_regex(self):
    comp = Regex('[wf][io]{2}')
    self.assertTrue( comp.test('fii') )
    self.assertTrue( comp.test('woo') )
    self.assertFalse( comp.test('fuu') )

  def test_regex_repr(self):
    comp = Regex('[wf][io]{2}')
    self.assertEquals(repr(comp), "Regex(pattern: [wf][io]{2}, flags: 0)")

  def test_any(self):
    comp = Any(1,2.3,str)
    self.assertTrue( comp.test(1) )
    self.assertTrue( comp.test(2.3) )
    self.assertFalse( comp.test(4) )
  
  def test_any_repr(self):
    comp = Any(1,2,3,str)
    self.assertEquals(repr(comp), "Any([1, 2, 3, IsA(str)])")
  
  def test_in(self):
    comp = In(['foo', 'bar'])
    self.assertTrue( comp.test('foo') )
    self.assertTrue( comp.test('bar') )
    self.assertFalse( comp.test('none') )
  
  def test_in_repr(self):
    comp = In(['foo', 'bar'])
    self.assertEqual(repr(comp), "In(['foo', 'bar'])")

  def test_contains(self):
    comp = Contains('foo')
    self.assertTrue( comp.test('foobar') )
    self.assertTrue( comp.test(['foo','bar']) )
    self.assertTrue( comp.test({'foo':'bar'}) )
    self.assertFalse( comp.test('feet') )
  
  def test_contains_repr(self):
    comp = Contains("foo")
    self.assertEqual(repr(comp), "Contains('foo')")

  def test_all(self):
    comp = All(IsA(bytearray), Equals('foo'))
    self.assertTrue( comp.test(bytearray('foo')) )
    self.assertFalse( comp.test('foo') )
    self.assertEquals( 'foo', bytearray('foo') )
  
  def test_all_repr(self):
    comp = All(IsA(bytearray), Equals('foobar'))
    self.assertEqual(repr(comp), "All([IsA(bytearray), 'foobar'])")

  def test_not(self):
    comp = Not( Any(1,3) )
    self.assertTrue( comp.test(2) )
    self.assertFalse( comp.test(1) )
    self.assertFalse( comp.test(3) )
  
  def test_no_repr(self):
    comp = Not(Any(1,3))
    self.assertEqual(repr(comp), "Not([Any([1, 3])])")

  def test_function(self):
    r = [True,False]
    comp = Function(lambda arg: r[arg])
    self.assertTrue( comp.test(0) )
    self.assertFalse( comp.test(1) )
  
  def test_function_repr(self):
    func = lambda arg: True
    comp = Function(func)
    self.assertEqual(repr(comp), "Function(%s)" % str(func))

  def test_ignore(self):
    comp = Ignore()
    self.assertTrue( comp.test('srsly?') )
  
  def test_ignore_repr(self):
    comp = Ignore()
    self.assertEqual(repr(comp), "Ignore()")

  def test_variable(self):
    comp = Variable('foo')
    self.assertEquals( 0, len(Variable._cache) )
    self.assertTrue( comp.test('bar') )
    self.assertEquals( 1, len(Variable._cache) )
    self.assertTrue( comp.test('bar') )
    self.assertFalse( comp.test('bar2') )
    
    self.assertTrue( Variable('foo').test('bar') )
    self.assertFalse( Variable('foo').test('bar2') )
    self.assertEquals( 1, len(Variable._cache) )

    self.assertEquals( 'bar', comp.value )
    self.assertEquals( 'bar', Variable('foo').value )

    v = Variable('foo2')
    self.assertEquals( 1, len(Variable._cache) )
    v.test('dog')
    self.assertEquals( 'dog', v.value )
    self.assertEquals( 2, len(Variable._cache) )

    Variable.clear()
    self.assertEquals( 0, len(Variable._cache) )

  def test_variable_repr(self):
    v = Variable('foo')
    self.assertEquals( repr(v), "Variable('foo')" )

  def test_like_init(self):
    c = Like({'foo':'bar'})
    self.assertEquals( {'foo':'bar'}, c._src )

    c = Like(['foo', 'bar'])
    self.assertEquals( ['foo','bar'], c._src )

  def test_like_test(self):
    c = Like({'foo':'bar'})
    self.assertTrue( c.test({'foo':'bar'}) )
    self.assertTrue( c.test({'foo':'bar', 'cat':'dog'}) )
    self.assertFalse( c.test({'foo':'barf'}) )
    
    c = Like(['foo','bar'])
    self.assertTrue( c.test(['foo','bar']) )
    self.assertTrue( c.test(['foo','bar','cat','dog']) )
    self.assertFalse( c.test(['foo','barf']) )

  def test_like_repr(self):
    c = Like({'foo':'bar'})
    self.assertEquals( repr(c), "Like({'foo': 'bar'})" )



import re

'''
All of the comparators that can be used for arguments.
'''

def build_comparators(*values_or_types):
  comparators = []
  for item in values_or_types:
    if isinstance(item,Comparator):
      comparators.append( item )
    elif isinstance(item,type):
      # If you are passing around a type you will have to build a Equals comparator
      comparators.append( IsA(item) )
    else:
      comparators.append( Equals(item) )
  return comparators

class Comparator(object):
  '''
  Base class of all comparators, used for type testing
  '''
  def __eq__(self, value):
    return self.test(value)

class Equals(Comparator):
  '''
  Simplest comparator.
  '''
  def __init__(self, value):
    self._value = value

  def test(self, value):
    return self._value == value
	
  def __repr__(self):
    return repr(self._value)
  __str__ = __repr__

class IsA(Comparator):
  '''
  Test to see if a value is an instance of something. Arguments match
  isinstance
  '''
  def __init__(self, types):
    self._types = types

  def test(self, value):
    return isinstance(value, self._types)
  
  def _format_name(self):
    if isinstance(self._types, type):
      return self._types.__name__
    else:
      return str([o.__name__ for o in self._types])
  
  def __repr__(self):
    return "IsA(%s)" % (self._format_name())
  __str__ = __repr__

class Is(Comparator):
  '''
  Checks for identity not equality
  '''
  def __init__(self, obj):
    self._obj = obj
    
  def test(self, value):
    return self._obj is value

  def __repr__(self):
    return "Is(%s)" % (str(self._obj))
  __str__ = __repr__

class AlmostEqual(Comparator):
  '''
  Compare a float value to n number of palces
  '''
  
  def __init__(self, float_value, places=7):
    self._float_value = float_value
    self._places = places
  
  def test(self, value):
    return round(value - self._float_value, self._places) == 0

  def __repr__(self):
    return "AlmostEqual(value: %s, places: %d)" % (str(self._float_value), self._places)
  __str__ = __repr__

class Regex(Comparator):
  '''
  Checks to see if a string matches a regex
  '''
  
  def __init__(self, pattern, flags=0):
    self._pattern = pattern
    self._flags = flags
    self._regex = re.compile(pattern)

  def test(self, value):
    return self._regex.search(value) is not None

  def __repr__(self):
    return "Regex(pattern: %s, flags: %s)" % (self._pattern, self._flags)
  __str__ = __repr__

class Any(Comparator):
  '''
  Test to see if any comparator matches
  '''
  def __init__(self, *comparators):
    self._comparators = build_comparators(*comparators)

  def test(self, value):
    for comp in self._comparators:
      if comp.test(value): return True
    return False

  def __repr__(self):
    return "Any(%s)" % str(self._comparators)
  __str__ = __repr__
  
class In(Comparator):
  '''
  Test if a key is in a list or dict
  '''
  def __init__(self, hay_stack):
    self._hay_stack = hay_stack

  def test(self, needle):
    return needle in self._hay_stack

  def __repr__(self):
    return "In(%s)" % (str(self._hay_stack))
  __str__ = __repr__

class Contains(Comparator):
  '''
  Test if a key is in a list or dict
  '''
  def __init__(self, needle):
    self._needle = needle

  def test(self, hay_stack):
    return self._needle in hay_stack

  def __repr__(self):
    return "Contains('%s')" % (str(self._needle))
  __str__ = __repr__

class All(Comparator):
  '''
  Test to see if all comparators match
  '''
  def __init__(self, *comparators):
    self._comparators = build_comparators(*comparators)

  def test(self, value):
    for comp in self._comparators:
      if not comp.test(value): return False
    return True

  def __repr__(self):
    return "All(%s)" % (self._comparators)
  __str__ = __repr__

class Not(Comparator):
  '''
  Return the opposite of a comparator
  '''
  def __init__(self, *comparators):
    self._comparators = build_comparators(*comparators)

  def test(self, value):
    return all([not c.test(value) for c in self._comparators])

  def __repr__(self):
    return "Not(%s)" % (repr(self._comparators))
  __str__ = __repr__

class Function(Comparator):
  '''
  Call a func to compare the values
  '''
  def __init__(self, func):
    self._func = func

  def test(self, value):
    return self._func(value)

  def __repr__(self):
    return "Function(%s)" % (str(self._func))
  __str__ = __repr__

class Ignore(Comparator):
  '''
  Igore this argument
  '''
  def test(self, value):
    return True

  def __repr__(self):
    return "Ignore()"
  __str__ = __repr__

class Variable(Comparator):
  '''
  A mechanism for tracking variables and their values.
  '''
  _cache = {}

  @classmethod
  def clear(self):
    '''
    Delete all cached values. Should only be used by the test suite.
    '''
    self._cache.clear()
  
  def __init__(self, name):
    self._name = name

  @property
  def value(self):
    try: return self._cache[self._name]
    except KeyError: raise ValueError("no value '%s'"%(self._name))

  def test(self, value):
    try:
      return self._cache[self._name] == value
    except KeyError:
      self._cache[self._name] = value
    return True

  def __repr__(self):
    return "Variable('%s')"%(self._name)
  __str__ = __repr__

class Like(Comparator):
  '''
  A comparator that will assert that fields of a container look like
  another.
  '''

  def __init__(self, src):
    # This might have to change to support more iterable types
    if not isinstance(src, (dict,set,list,tuple)):
      raise ValueError("Like comparator only implemented for basic container types")
    self._src = src

  def test(self, value):
    # This might need to change so that the ctor arg can be a list, but
    # any iterable type can be tested.
    if not isinstance(value, type(self._src)):
      return False

    rval = True
    if isinstance(self._src, dict):
      for k,v in self._src.iteritems():
        rval = rval and value.get(k)==v

    else:
      for item in self._src:
        rval = rval and item in value

    return rval

  def __repr__(self):
    return "Like(%s)"%(str(self._src))
  __str__ = __repr__



'''
Implementation of stubbing
'''
import inspect
import types
import os
import sys
import gc

from expectation import Expectation, ArgumentsExpectationRule
from exception import *
from termcolor import colored

# For clarity here and in tests, could make these class or static methods on
# Stub. Chai base class would hide that.
def stub(obj, attr=None):
  '''
  Stub an object. If attr is not None, will attempt to stub that attribute
  on the object. Only required for modules and other rare cases where we
  can't determine the binding from the object.
  '''
  if attr:
    return _stub_attr(obj, attr)
  else:
    return _stub_obj(obj)

def _stub_attr(obj, attr_name):
  '''
  Stub an attribute of an object. Will return an existing stub if there already
  is one.
  '''
  # Annoying circular reference requires importing here. Would like to see
  # this cleaned up. @AW
  from mock import Mock

  # Check to see if this a property, this check is only for when dealing with an
  # instance. getattr will work for classes.
  is_property = False

  if not inspect.isclass(obj) and not inspect.ismodule(obj):
    # It's possible that the attribute is defined after initialization, and
    # so is not on the class itself.
    attr = getattr(obj.__class__, attr_name, None)
    if isinstance(attr, property):
      is_property = True

  if not is_property:
    attr = getattr(obj, attr_name)

  # Return an existing stub
  if isinstance(attr, Stub):
    return attr

  # If a Mock object, stub its __call__
  if isinstance(attr, Mock):
    return stub(attr.__call__)

  if isinstance(attr, property):
    return StubProperty(obj, attr_name)

  # Sadly, builtin functions and methods have the same type, so we have to use
  # the same stub class even though it's a bit ugly
  if inspect.ismodule(obj) and \
  isinstance(attr, (types.FunctionType,types.BuiltinFunctionType,types.BuiltinMethodType)):
    return StubFunction(obj, attr_name)

  if isinstance(attr, types.MethodType):
    # Handle differently if unbound because it's an implicit "any instance"
    if attr.im_self==None:
      return StubUnboundMethod(attr)
    else:
      return StubMethod(obj, attr_name)

  if isinstance(attr, (types.BuiltinFunctionType,types.BuiltinMethodType)):
    return StubFunction(obj, attr_name)

  # What an absurd type this is ....
  if type(attr).__name__ == 'method-wrapper':
    return StubMethodWrapper(attr)

  # This is also slot_descriptor
  if type(attr).__name__ == 'wrapper_descriptor':
    return StubWrapperDescriptor(obj, attr_name)

  raise UnsupportedStub("can't stub %s(%s) of %s", attr_name, type(attr), obj)


def _stub_obj(obj):
  '''
  Stub an object directly.
  '''
  # Annoying circular reference requires importing here. Would like to see
  # this cleaned up. @AW
  from mock import Mock

  # Return an existing stub
  if isinstance(obj, Stub):
    return obj

  # If a Mock object, stub its __call__
  if isinstance(obj, Mock):
    return stub(obj.__call__)

  # If passed-in a type, assume that we're going to stub out the creation.
  # See StubNew for the awesome sauce.
  if isinstance(obj, types.TypeType):
    return StubNew(obj)

  # I thought that types.UnboundMethodType differentiated these cases but
  # apparently not.
  if isinstance(obj, types.MethodType):
    # Handle differently if unbound because it's an implicit "any instance"
    if obj.im_self==None:
      return StubUnboundMethod(obj)
    else:
      return StubMethod(obj)

  # These aren't in the types library
  if type(obj).__name__ == 'method-wrapper':
    return StubMethodWrapper(obj)

  if type(obj).__name__ == 'wrapper_descriptor':
    raise UnsupportedStub("must call stub(obj,'%s') for slot wrapper on %s",
      obj.__name__, obj.__objclass__.__name__ )

  # (Mostly) Lastly, look for properties.
  # First look for the situation where there's a reference back to the property.
  prop = obj
  if isinstance( getattr( obj, '__self__', None), property ):
    obj = prop.__self__

  # Once we've found a property, we have to figure out how to reference back to
  # the owning class. This is a giant pain and we have to use gc to find out
  # where it comes from. This code is dense but resolves to something like this:
  # >>> gc.get_referrers( foo.x )
  # [{'__dict__': <attribute '__dict__' of 'foo' objects>,
  #   'x': <property object at 0x7f68c99a16d8>,
  #   '__module__': '__main__',
  #   '__weakref__': <attribute '__weakref__' of 'foo' objects>,
  #   '__doc__': None}]
  if isinstance(obj, property):
    klass,attr = None,None
    for ref in gc.get_referrers( obj ):
      if klass and attr: break
      if isinstance(ref,dict) and ref.get('prop',None) is obj :
        klass = getattr( ref.get('__dict__',None), '__objclass__', None )
        for name,val in getattr(klass,'__dict__',{}).iteritems():
          if val is obj:
            attr = name
            break

    if klass and attr:
      rval = stub(klass,attr)
      if prop != obj:
        return stub(rval, prop.__name__)
      return rval

  # If a function and it has an associated module, we can mock directly.
  # Note that this *must* be after properties, otherwise it conflicts with
  # stubbing out the deleter methods and such
  # Sadly, builtin functions and methods have the same type, so we have to use
  # the same stub class even though it's a bit ugly
  if isinstance(obj, (types.FunctionType,types.BuiltinFunctionType,types.BuiltinMethodType)) and hasattr(obj, '__module__'):
    return StubFunction(obj)

  raise UnsupportedStub("can't stub %s", obj)

class Stub(object):
  '''
  Base class for all stubs.
  '''

  def __init__(self, obj, attr=None):
    '''
    Setup the structs for expectations
    '''
    self._obj = obj
    self._attr = attr
    self._expectations = []

  @property
  def name(self):
    return None # The base class implement this.

  def unmet_expectations(self):
    '''
    Assert that all expectations on the stub have been met.
    '''
    unmet = []
    for exp in self._expectations:
      if not exp.closed(with_counts=True):
        unmet.append(ExpectationNotSatisfied(exp))
    return unmet

  def teardown(self):
    '''
    Clean up all expectations and restore the original attribute of the mocked
    object.
    '''
    self._expectations = []

  def expect(self):
    '''
    Add an expectation to this stub. Return the expectation
    '''
    exp = Expectation(self)
    self._expectations.append( exp )
    return exp

  def __call__(self, *args, **kwargs):
    for exp in self._expectations:
      # If expectation closed skip
      if exp.closed():
        continue

      # If args don't match the expectation, close it and move on, else
      # pass to it for testing.
      if not exp.match(*args, **kwargs):
        exp.close(*args, **kwargs)
      else:
        return exp.test(*args, **kwargs)

    raise UnexpectedCall("\n\n" + self._format_exception(ArgumentsExpectationRule.pretty_format_args(*args, **kwargs)))

  def _format_exception(self, args_str):
    result = [
      colored("No expectation in place for %s with %s" % (self.name, args_str), "red"),
      "All Expectations:"
    ]
    for exception in self._expectations:
      result.append(str(exception))

    return "\n".join(result)

class StubProperty(Stub, property):
  '''
  Property stubbing.
  '''

  def __init__(self, obj, attr):
    super(StubProperty,self).__init__(obj, attr)
    property.__init__(self, lambda x: self(),
      lambda x, val: self.setter(val), lambda x: self.deleter() )
    # In order to stub out a property we have ask the class for the propery object
    # that was created we python execute class code.
    if inspect.isclass(obj):
      self._instance = obj
    else:
      self._instance = obj.__class__

    # Use a simple Mock object for the deleter and setter. Use same namespace
    # as property type so that it simply works.
    # Annoying circular reference requires importing here. Would like to see
    # this cleaned up. @AW
    from mock import Mock
    self._obj = getattr(obj, attr)
    self.setter = Mock()
    self.deleter = Mock()

    setattr(self._instance, self._attr, self)

  @property
  def name(self):
    return "%s.%s" % (self._instance.__name__, self._attr)


  def teardown(self):
    '''
    Replace the original method.
    '''
    setattr( self._instance, self._attr, self._obj )

class StubMethod(Stub):
  '''
  Stub a method.
  '''

  def __init__(self, obj, attr=None):
    '''
    Initialize with an object of type MethodType
    '''
    super(StubMethod,self).__init__(obj, attr)
    if not self._attr:
      self._attr = obj.im_func.func_name
      self._instance = obj.im_self
    else:
      self._instance = self._obj
      self._obj = getattr( self._instance, self._attr )
    setattr( self._instance, self._attr, self )

  @property
  def name(self):
    from mock import Mock # Import here for the same reason as above.
    if hasattr(self._obj, 'im_class'):
      if issubclass(self._obj.im_class, Mock):
        return "%s (on mock object)" % self._obj.im_self._name

    # Always use the class to get the name
    klass = self._instance
    if not inspect.isclass(self._instance):
      klass = self._instance.__class__

    return "%s.%s" % (klass.__name__, self._attr)

  def teardown(self):
    '''
    Put the original method back in place. This will also handle the special case
    when it putting back a class method.

    The following code snippet best describe why it fails using settar, the
    class method would be replaced with a bound method not a class method.

    >>> class Example(object):
    ...     @classmethod
    ...     def a_classmethod(self):
    ...         pass
    ...
    >>> Example.__dict__['a_classmethod'] # Note the classmethod is returned.
    <classmethod object at 0x7f5e6c298be8>
    >>> orig = getattr(Example, 'a_classmethod')
    >>> orig
    <bound method type.a_classmethod of <class '__main__.Example'>>
    >>> setattr(Example, 'a_classmethod', orig)
    >>> Example.__dict__['a_classmethod'] # Note that setattr set a bound method not a class method.
    <bound method type.a_classmethod of <class '__main__.Example'>>

    The only way to figure out if this is a class method is to check and see if
    the bound method im_self is a class, if so then we need to wrap the function
    object (im_func) with class method before setting it back on the class.

    '''
    # Figure out if this is a class method and we're unstubbing it on the class
    # to which it belongs. This addresses an edge case where a module can 
    # expose a method of an instance. gevent does this, for example.
    if inspect.isclass(self._obj.im_self) and self._obj.im_self is self._instance:
      # Wrap it and set it back on the class
      setattr(self._instance, self._attr, classmethod(self._obj.im_func))
    else:
      setattr( self._instance, self._attr, self._obj )

class StubFunction(Stub):
  '''
  Stub a function.
  '''

  def __init__(self, obj, attr=None):
    '''
    Initialize with an object that is an unbound method
    '''
    super(StubFunction, self).__init__(obj, attr)
    if not self._attr:
      if getattr(obj, '__module__', None):
        self._instance = sys.modules[obj.__module__]
      elif getattr(obj, '__self__', None):
        self._instance = obj.__self__
      else:
        raise UnsupportedStub("Failed to find instance of %s"%(obj))

      if getattr(obj,'func_name', None):
        self._attr = obj.func_name
      elif getattr(obj,'__name__', None):
        self._attr = obj.__name__
      else:
        raise UnsupportedStub("Failed to find name of %s"%(obj))
    else:
      self._instance = self._obj
      self._obj = getattr(self._instance, self._attr)

    # This handles the case where we're stubbing a special method that's
    # inherited from object, and so instead of calling setattr on teardown,
    # we want to call delattr. This is particularly important for not seeing
    # those stupid DeprecationWarnings after StubNew
    self._was_object_method = False
    if hasattr(self._instance, '__dict__'):
      self._was_object_method = \
        self._attr not in self._instance.__dict__.keys() and\
        self._attr in object.__dict__.keys()
    setattr( self._instance, self._attr, self )

  @property
  def name(self):
    return "%s.%s" % (self._instance.__name__, self._attr)

  def teardown(self):
    '''
    Replace the original method.
    '''
    if not self._was_object_method:
      setattr( self._instance, self._attr, self._obj )
    else:
      delattr( self._instance, self._attr )

class StubNew(StubFunction):
  '''
  Stub out the constructor, but hide the fact that we're stubbing "__new__"
  and act more like we're stubbing "__init__". Needs to use the logic in
  the StubFunction ctor.
  '''
  _cache = {}

  def __new__(self, klass, *args):
    '''
    Because we're not saving the stub into any attribute, then we have
    to do some faking here to return the same handle.
    '''
    rval = self._cache.get(klass)
    if not rval:
      rval = self._cache[klass] = super(StubNew,self).__new__(self, *args)
    return rval

  def __init__(self, obj):
    '''
    Overload the initialization so that we can hack access to __new__.
    '''
    super(StubNew,self).__init__(obj, '__new__')
    self._type = obj

  def __call__(self, *args, **kwargs):
    '''
    When calling the new function, strip out the first arg which is
    the type. In this way, the mocker writes their expectation as if it
    was an __init__.
    '''
    return super(StubNew,self).__call__( *(args[1:]), **kwargs )
  
  def teardown(self):
    '''
    Overload so that we can clear out the cache after a test run.
    '''
    super(StubNew,self).teardown()
    StubNew._cache.pop(self._type)

class StubUnboundMethod(Stub):
  '''
  Stub an unbound method.
  '''

  def __init__(self, obj):
    '''
    Initialize with an object that is an unbound method
    '''
    super(StubUnboundMethod,self).__init__(obj)
    self._instance = obj.im_class
    self._attr = obj.im_func.func_name
    setattr( self._instance, self._attr, self )

  @property
  def name(self):
    return "%s.%s" % (self._instance.__name__, self._attr)

  def teardown(self):
    '''
    Replace the original method.
    '''
    setattr( self._instance, self._attr, self._obj )

class StubMethodWrapper(Stub):
  '''
  Stub a method-wrapper.
  '''

  def __init__(self, obj):
    '''
    Initialize with an object that is a method wrapper.
    '''
    super(StubMethodWrapper,self).__init__(obj)
    self._instance = obj.__self__
    self._attr = obj.__name__
    setattr( self._instance, self._attr, self )

  @property
  def name(self):
    return "%s.%s" % (self._instance.__class__.__name__, self._attr)

  def teardown(self):
    '''
    Replace the original method.
    '''
    setattr( self._instance, self._attr, self._obj )

class StubWrapperDescriptor(Stub):
  '''
  Stub a wrapper-descriptor. Only works when we can fetch it by name. Because
  the w-d object doesn't contain both the instance ref and the attribute name
  to be able to look it up. Used for mocking object.__init__ and related
  builtin methods when subclasses that don't overload those.
  '''

  def __init__(self, obj, attr_name):
    '''
    Initialize with an object that is a method wrapper.
    '''
    super(StubWrapperDescriptor,self).__init__(obj, attr_name)
    self._orig = getattr( self._obj, self._attr )
    setattr( self._obj, self._attr, self )

  @property
  def name(self):
    return "%s.%s" % (self._obj.__name__, self._attr)

  def teardown(self):
    '''
    Replace the original method.
    '''
    setattr( self._obj, self._attr, self._orig )




from chai import Chai
import socket
import datetime

######################################
## Mocking sockets
######################################
def connect():
  sock = socket.socket()
  sock.bind(('127.0.0.1', 10000))
  return sock.recv(1024)

def get_host():
  return socket.gethostname()

class SocketTestCase(Chai):
    
  def test_socket(self):
    mock_socket = self.mock()
    self.mock(socket, 'socket')
    self.expect(socket.socket.__call__).returns(mock_socket)
    
    self.expect(mock_socket.bind).args(('127.0.0.1' , 10000))
    self.expect(mock_socket.recv).args(1024).returns("HELLO WORLD")
    
    self.assert_equals(connect(), "HELLO WORLD")

  def test_get_host(self):
    self.mock(socket, 'gethostname')
    self.expect(socket.gethostname.__call__).returns("my_host")
    self.assert_equals(get_host(), "my_host")


######################################
## Mocking datetime.now
######################################
def now():
  return datetime.datetime.now()

class DatetimeTestCase(Chai):
    
    def test_now(self):
      current_now = datetime.datetime.now() # Save for later

      # NOTE: In c python you are not allow to mock built types so we have to mock
      # the entire module.moc
      mock_datetime = self.mock(datetime, 'datetime')
      self.expect(mock_datetime.now).returns(current_now)
      
      self.assert_equals(now(), current_now)

if __name__ == '__main__':
  import unittest2
  unittest2.main()





import unittest
from collections import deque

from chai import Chai
from chai.chai import ChaiTestType
from chai.mock import Mock
from chai.stub import Stub
from chai.exception import *
from chai.comparators import Comparator

class CupOf(Chai):
  '''
  An example of a subclass on which we can test certain features.
  '''

  class msg_equals(Comparator):
    '''
    A Comparator used for check message equality
    '''
    def __init__(self, (key,value)):
      self._key = key
      self._value = value

    def test(self, value):
      if isinstance(value,dict):
        return self._value==value.get(self._key)
      return False

  def assert_msg_sent(self, msg):
    '''
    Assert that a message was sent and marked that it was handled.
    '''
    return msg.get('sent_at') and msg.get('received')

  def test_local_definitions_work_and_are_global(self):
    class Foo(object):
      def _save_data(self, msg):
        pass #dosomethingtottalyawesomewiththismessage
        
      def do_it(self, msg):
        self._save_data(msg)
        msg['sent_at'] = 'now'
        msg['received'] = 'yes'

    f = Foo()
    expect( f._save_data ).args( msg_equals(('target','bob')) )
    
    msg = {'target':'bob'}
    f.do_it( msg )
    assert_msg_sent( msg )

  def test_something(self): pass
  def runTest(self, *args, **kwargs): pass

class ChaiTest(unittest.TestCase):

  def test_init(self):
    case = CupOf.__new__(CupOf)
    self.assertTrue( hasattr(case, 'assertEquals') )
    self.assertFalse( hasattr(case, 'assert_equals') )
    case.__init__()
    self.assertTrue( hasattr(case, 'assertEquals') )
    self.assertTrue( hasattr(case, 'assert_equals') )

  def test_setup(self):
    case = CupOf()
    case.setup()
    self.assertEquals( deque(), case._stubs )
    self.assertEquals( deque(), case._mocks )

  def test_teardown_closes_out_stubs_and_mocks(self):
      class Stub(object):
        calls = 0
        def teardown(self): self.calls += 1
    
      obj = type('test',(object,),{})()
      setattr(obj, 'mock1', 'foo')
      setattr(obj, 'mock2', 'bar')
      
      case = CupOf()
      stub = Stub()
      case._stubs = deque([stub])
      case._mocks = deque([(obj,'mock1','fee'), (obj,'mock2')])
      case.teardown()
      self.assertEquals( 1, stub.calls )
      self.assertEquals( 'fee', obj.mock1 )
      self.assertFalse( hasattr(obj, 'mock2') )

  def test_stub(self):
    class Milk(object):
      def pour(self): pass

    case = CupOf()
    milk = Milk()
    case.setup()
    self.assertEquals( deque(), case._stubs )
    case.stub( milk.pour )
    self.assertTrue( isinstance(milk.pour, Stub) )
    self.assertEquals( deque([milk.pour]), case._stubs )

    # Test it's only added once
    case.stub( milk, 'pour' )
    self.assertEquals( deque([milk.pour]), case._stubs )

  def test_expect(self):
    class Milk(object):
      def pour(self): pass

    case = CupOf()
    milk = Milk()
    case.setup()
    self.assertEquals( deque(), case._stubs )
    case.expect( milk.pour )
    self.assertEquals( deque([milk.pour]), case._stubs )

    # Test it's only added once
    case.expect( milk, 'pour' )
    self.assertEquals( deque([milk.pour]), case._stubs )

    self.assertEquals( 2, len(milk.pour._expectations) )

  def test_mock_no_binding(self):
    case = CupOf()
    case.setup()

    self.assertEquals( deque(), case._mocks )
    mock1 = case.mock()
    self.assertTrue( isinstance(mock1, Mock) )
    self.assertEquals( deque(), case._mocks )
    mock2 = case.mock()
    self.assertTrue( isinstance(mock2, Mock) )
    self.assertEquals( deque(), case._mocks )
    self.assertNotEqual( mock1, mock2 )

  def test_mock_with_attr_binding(self):
    class Milk(object):
      def __init__(self): self._case = []
      def pour(self): return self._case.pop(0)

    case = CupOf()
    case.setup()
    milk = Milk()
    orig_pour = milk.pour

    self.assertEquals( deque(), case._mocks )
    mock1 = case.mock( milk, 'pour' )
    self.assertTrue( isinstance(mock1, Mock) )
    self.assertEquals( deque([(milk,'pour',orig_pour)]), case._mocks )
    mock2 = case.mock( milk, 'pour' )
    self.assertTrue( isinstance(mock2, Mock) )
    self.assertEquals( deque([(milk,'pour',orig_pour),(milk,'pour',mock1)]), case._mocks )
    self.assertNotEqual( mock1, mock2 )

    mock3 = case.mock( milk, 'foo' )
    self.assertTrue( isinstance(mock3, Mock) )
    self.assertEquals( deque([(milk,'pour',orig_pour),(milk,'pour',mock1),(milk,'foo')]), case._mocks )
    
  def test_chai_class_use_metaclass(self):
    obj = CupOf()    
    self.assertTrue(obj, ChaiTestType)
    
  def test_runs_unmet_expectations(self):
    class Stub(object):
      calls = 0
      def unmet_expectations(self): self.calls += 1; return []
      def teardown(self): self.calls += 1

    # obj = type('test',(object,),{})()
    # setattr(obj, 'mock1', 'foo')
    # setattr(obj, 'mock2', 'bar')
    
    case = CupOf()
    stub = Stub()
    case._stubs = deque([stub])
    
    case.test_local_definitions_work_and_are_global()
    self.assertEquals(stub.calls, 1)


import re
import unittest
import types

from chai.stub import *
from chai.mock import Mock
import samples

class StubTest(unittest.TestCase):
  ###
  ### Test the public stub() method
  ###
  def test_stub_property_on_class_with_attr_name(self):
    class Foo(object):
      @property
      def prop(self): return 3
    
    res = stub(Foo, 'prop')
    self.assertTrue( isinstance(res,StubProperty) )
    self.assertTrue( stub(Foo,'prop') is res )

  def test_stub_property_on_instance_with_attr_name(self):
    class Foo(object):
      @property
      def prop(self): return 3
    foo = Foo()
    
    res = stub(foo, 'prop')
    self.assertTrue( isinstance(res,StubProperty) )
    self.assertTrue( stub(foo,'prop') is res )

  def test_stub_property_on_class_with_attr_name_applies_to_instance(self):
    class Foo(object):
      @property
      def prop(self): return 3
    
    foo = Foo()
    res = stub(Foo, 'prop')
    self.assertTrue( stub(foo,'prop') is res )

  def test_stub_property_with_obj_ref_for_the_reader(self):
    class Foo(object):
      @property
      def prop(self): return 3

    res = stub(Foo.prop)
    self.assertTrue( isinstance(res, StubProperty) )
    self.assertTrue( stub(Foo.prop) is res )

  def test_stub_property_with_obj_ref_for_the_setter(self):
    class Foo(object):
      @property
      def prop(self): return 3

    res = stub(Foo.prop.setter)
    self.assertTrue( isinstance(res, StubMethod) )
    self.assertTrue( isinstance(Foo.prop, StubProperty) )

  def test_stub_property_with_obj_ref_for_the_deleter(self):
    class Foo(object):
      @property
      def prop(self): return 3

    res = stub(Foo.prop.deleter)
    self.assertTrue( isinstance(res, StubMethod) )
    self.assertTrue( isinstance(Foo.prop, StubProperty) )

  def test_stub_mock_with_attr_name(self):
    class Foo(object):
      def bar(self): pass

    f = Foo()
    f.bar = Mock()
    res = stub(f, 'bar')
    self.assertTrue( isinstance(res, StubMethod) )
    self.assertEquals( res, f.bar.__call__ )
    self.assertEquals( res, stub(f, 'bar') )

  def test_stub_mock_with_obj_ref(self):
    class Foo(object):
      def bar(self): pass

    f = Foo()
    f.bar = Mock()
    res = stub(f.bar)
    self.assertTrue( isinstance(res, StubMethod) )
    self.assertEquals( res, f.bar.__call__ )
    self.assertEquals( res, stub(f.bar) )

  def test_stub_type_with_obj_ref(self):
    class Foo(object):
      def bar(self): pass

    res = stub( Foo )
    self.assertTrue( isinstance(res, StubNew) )
    self.assertEquals( res, Foo.__new__ )
    self.assertEquals( res, stub(Foo) )

  def test_stub_unbound_method_with_attr_name(self):
    class Foo(object):
      def bar(self): pass

    res = stub(Foo, 'bar')
    self.assertTrue( isinstance(res,StubUnboundMethod) )
    self.assertEquals( res, stub(Foo,'bar') )
    self.assertEquals( res, getattr(Foo,'bar') )

  def test_stub_unbound_method_with_obj_ref(self):
    class Foo(object):
      def bar(self): pass

    res = stub(Foo.bar)
    self.assertTrue( isinstance(res,StubUnboundMethod) )
    self.assertEquals( res, stub(Foo.bar) )
    self.assertEquals( res, getattr(Foo,'bar') )

  def test_stub_bound_method_for_instance_with_attr_name(self):
    class Foo(object):
      def bar(self): pass

    foo = Foo()
    orig = foo.bar
    res = stub(foo, 'bar')
    self.assertTrue( isinstance(res,StubMethod) )
    self.assertEquals( res._instance, foo )
    self.assertEquals( res._obj, orig )
    self.assertEquals( res._attr, 'bar' )
    self.assertEquals( res, stub(foo,'bar') )
    self.assertEquals( res, getattr(foo,'bar') )

  def test_stub_bound_method_for_instance_with_obj_ref(self):
    class Foo(object):
      def bar(self): pass

    foo = Foo()
    orig = foo.bar
    res = stub(foo.bar)
    self.assertTrue( isinstance(res,StubMethod) )
    self.assertEquals( res._instance, foo )
    self.assertEquals( res._obj, orig )
    self.assertEquals( res._attr, 'bar' )
    self.assertEquals( res, stub(foo.bar) )
    self.assertEquals( res, getattr(foo,'bar') )

  def test_stub_bound_method_for_classmethod_with_attr_name(self):
    class Foo(object):
      @classmethod
      def bar(self): pass

    res = stub(Foo, 'bar')
    self.assertTrue( isinstance(res,StubMethod) )
    self.assertEquals( res, stub(Foo,'bar') )
    self.assertEquals( res, getattr(Foo,'bar') )

  def test_stub_bound_method_for_classmethod_with_obj_ref(self):
    class Foo(object):
      @classmethod
      def bar(self): pass

    res = stub(Foo.bar)
    self.assertTrue( isinstance(res,StubMethod) )
    self.assertEquals( res, stub(Foo.bar) )
    self.assertEquals( res, getattr(Foo,'bar') )

  def test_stub_method_wrapper_with_attr_name(self):
    class Foo(object): pass

    foo = Foo()
    res = stub(foo, '__hash__')
    self.assertTrue( isinstance(res,StubMethodWrapper) )
    self.assertEquals( res, stub(foo, '__hash__') )
    self.assertEquals( res, getattr(foo, '__hash__') )

  def test_stub_method_wrapper_with_obj_ref(self):
    class Foo(object): pass

    foo = Foo()
    res = stub(foo.__hash__)
    self.assertTrue( isinstance(res,StubMethodWrapper) )
    self.assertEquals( res, stub(foo.__hash__) )
    self.assertEquals( res, getattr(foo, '__hash__') )

  def test_stub_module_function_with_attr_name(self):
    res = stub(samples, 'mod_func_1')
    self.assertTrue( isinstance(res,StubFunction) )
    self.assertEquals( res, getattr(samples,'mod_func_1') )
    self.assertEquals( res, stub(samples,'mod_func_1') )

  def test_stub_module_function_with_obj_ref(self):
    res = stub(samples.mod_func_1) 
    self.assertTrue( isinstance(res,StubFunction) )
    self.assertEquals( res, getattr(samples,'mod_func_1') )
    self.assertEquals( res, samples.mod_func_1 )
    self.assertEquals( res, stub(samples.mod_func_1) )
  
class StubClassTest(unittest.TestCase):
  ###
  ### Test Stub class (if only I could mock my mocking mocks)
  ###
  def test_init(self):
    s = Stub('obj','attr')
    self.assertEquals( 'obj', s._obj )
    self.assertEquals( 'attr', s._attr )
    self.assertEquals( [], s._expectations )

  def test_unment_expectations(self):
    s = Stub('obj', 'attr')
    s.expect().args(123).returns(1)
    
    self.assertTrue(all([isinstance(e, ExpectationNotSatisfied) for e in s.unmet_expectations()]))
  
  def test_teardown(self):
    s = Stub('obj')
    s._expections = ['1','2']
    s.teardown()
    self.assertEquals( [], s._expectations )

  def test_expect(self):
    s = Stub('obj')

    self.assertEquals( [], s._expectations )
    e = s.expect()
    self.assertEquals( [e], s._expectations )
    self.assertEquals( s, e._stub )

  def test_call_raises_unexpected_call_when_no_expectations(self):
    s = Stub('obj')
    self.assertRaises( UnexpectedCall, s, 'foo' )

  def test_call_when_args_match(self):
    class Expect(object):
      def closed(self): return False
      def match(self, *args, **kwargs): return True
      def test(self, *args, **kwargs): return 'success'
    
    s = Stub('obj')
    s._expectations = [ Expect() ]
    self.assertEquals( 'success', s('foo') )
  
  def test_call_raises_unexpected_call_when_all_expectations_closed(self):
    class Expect(object):
      def closed(self): return True
    
    s = Stub('obj')
    s._expectations = [ Expect(), Expect() ]
    self.assertRaises( UnexpectedCall, s, 'foo' )

  def test_call_raises_unexpected_call_when_closed_and_no_matching(self):
    class Expect(object):
      def __init__(self, closed): 
        self._closed=closed
        self._match_count = 0
        self._close_count=0
      def closed(self):
        return self._closed
      def match(self, *args, **kwargs):
        self._match_count +=1
        return False
      def close(self, *args, **kwargs):
        self._close_count += 1
        self._close_args = (args,kwargs)
    
    s = Stub('obj')
    s._expectations = [ Expect(True), Expect(False) ]
    self.assertRaises( UnexpectedCall, s, 'foo' )
    self.assertEquals( 0, s._expectations[0]._match_count )
    self.assertEquals( 1, s._expectations[1]._match_count )
    self.assertEquals( 0, s._expectations[0]._close_count )
    self.assertEquals( 1, s._expectations[1]._close_count )
    self.assertEquals( (('foo',),{}), s._expectations[1]._close_args )
    

class StubPropertyTest(unittest.TestCase):
  # FIXME: Need to test teardown and init, these test might be in the base stub tests.
  
  def test_name(self):
    class Foo(object):
      @property
      def prop(self): return 3
    
    s = StubProperty(Foo, 'prop')
    self.assertEquals(s.name, 'Foo.prop')

class StubMethodTest(unittest.TestCase):
  
  def test_init(self):
    class Foo(object):
      def bar(self): pass

    f = Foo()
    orig = f.bar
    s = StubMethod( f.bar )
    self.assertEquals( s._obj, orig )
    self.assertEquals( s._instance, f )
    self.assertEquals( s._attr, 'bar' )
    self.assertEquals( s, getattr(f,'bar') )
    
    f = Foo()
    orig = f.bar
    s = StubMethod( f, 'bar' )
    self.assertEquals( s._obj, orig )
    self.assertEquals( s._instance, f )
    self.assertEquals( s._attr, 'bar' )
    self.assertEquals( s, getattr(f,'bar') )

  def test_name(self):
    class Expect(object):
      def closed(self): return False
    obj = Expect()
    s = StubMethod(obj.closed)
    self.assertEquals("Expect.closed", s.name)
    s.teardown()

    s = StubMethod(obj, 'closed')
    self.assertEquals("Expect.closed", s.name)
    s.teardown()

  def test_teardown(self):
    class Foo(object):
      def bar(self): pass

    f = Foo()
    orig = f.bar
    s = StubMethod( f.bar )
    s.teardown()
    self.assertEquals( orig, f.bar )

  def test_teardown_of_classmethods(self):
    class Foo(object):
      @classmethod
      def bar(self): pass
    
    self.assertTrue(isinstance(Foo.__dict__['bar'], classmethod))
    s = StubMethod( Foo.bar )
    s.teardown()
    self.assertTrue(isinstance(Foo.__dict__['bar'], classmethod), "Is not a classmethod")

  def test_teardown_of_bound_instance_methods_exported_in_module(self):
    orig = samples.mod_instance_foo
    s = StubMethod( samples, 'mod_instance_foo' )
    s.teardown()
    self.assertEquals( orig, samples.mod_instance_foo )

class StubFunctionTest(unittest.TestCase):

  def test_init(self):
    s = StubFunction( samples.mod_func_1 )
    self.assertEquals( s._instance, samples )
    self.assertEquals( s._attr, 'mod_func_1' )
    self.assertEquals( s, samples.mod_func_1 )
    self.assertEquals( False, s._was_object_method )
    s.teardown()

  def test_init_with_object_method(self):
    x = samples.SampleBase()
    s = StubFunction( x, '__new__' )
    self.assertEquals( True, s._was_object_method )

  def test_name(self):
    s = StubFunction( samples.mod_func_1 )
    self.assertEquals( 'tests.samples.mod_func_1', s.name )
    s.teardown()

  def test_teardown(self):
    orig = samples.mod_func_1
    s = StubFunction( samples.mod_func_1 )
    s.teardown()
    self.assertEquals( orig, samples.mod_func_1 )

  def test_teardown_on_object_method(self):
    x = samples.SampleBase()
    self.assertEquals( object.__new__, getattr(x, '__new__') )
    s = StubFunction( x, '__new__' )
    self.assertNotEquals( object.__new__, getattr(x, '__new__') )
    s.teardown()
    self.assertEquals( object.__new__, getattr(x, '__new__') )

class StubNewTest(unittest.TestCase):

  def test_new(self):
    class Foo(object): pass

    self.assertEquals( 0, len(StubNew._cache) )
    x = StubNew(Foo)
    self.assertTrue( x is StubNew(Foo) )
    self.assertEquals( 1, len(StubNew._cache) )
    StubNew._cache.clear()

  def test_init(self):
    class Foo(object): pass
    
    s = StubNew( Foo )
    self.assertEquals( s._instance, Foo )
    self.assertEquals( s._attr, '__new__' )
    self.assertEquals( s, Foo.__new__ )
    s.teardown()

  def test_call(self):
    class Foo(object): pass
    class Expect(object):
      def closed(self): return False
      def match(self, *args, **kwargs): return args==('state',) and kwargs=={'a':'b'}
      def test(self, *args, **kwargs): return 'success'
    
    s = StubNew( Foo )
    s._expectations = [ Expect() ]
    self.assertEquals( 'success', Foo('state', a='b') )
    s.teardown()

  def test_teardown(self):
    class Foo(object): pass

    orig = Foo.__new__
    self.assertEquals( 0, len(StubNew._cache) )
    x = StubNew(Foo)
    self.assertEquals( 1, len(StubNew._cache) )
    x.teardown()
    self.assertEquals( 0, len(StubNew._cache) )
    self.assertEquals( orig, Foo.__new__ )

class StubUnboundMethodTest(unittest.TestCase):
  
  def test_init(self):
    class Foo(object):
      def bar(self): pass

    s = StubUnboundMethod( Foo.bar )
    self.assertEquals( s._instance, Foo )
    self.assertEquals( s._attr, 'bar' )
    self.assertEquals( s, getattr(Foo,'bar') )

  def test_name(self):
    class Expect(object):
      def closed(self): return False

    s = StubUnboundMethod(Expect.closed)
    self.assertEquals("Expect.closed", s.name)
    s.teardown()

  def test_teardown(self):
    class Foo(object):
      def bar(self): pass

    orig = Foo.bar
    s = StubUnboundMethod( Foo.bar )
    s.teardown()
    self.assertEquals( orig, Foo.bar )

  def test_call_acts_as_any_instance(self):
    class Foo(object):
      def bar(self): pass

    class StubIntercept(StubUnboundMethod):
      calls = 0
      def __call__(self, *args, **kwargs):
        self.calls += 1

    orig = Foo.bar
    s = StubIntercept( Foo.bar )

    f1 = Foo()
    f1.bar()
    f2 = Foo()
    f2.bar()

    self.assertEquals( 2, s.calls )

class StubMethodWrapperTest(unittest.TestCase):
  
  def test_init(self):
    class Foo(object):pass
    foo = Foo()

    s = StubMethodWrapper( foo.__hash__ )
    self.assertEquals( s._instance, foo )
    self.assertEquals( s._attr, '__hash__' )
    self.assertEquals( s, getattr(foo,'__hash__') )

  def test_name(self):
    class Foo(object):pass
    foo = Foo()

    s = StubMethodWrapper(foo.__hash__)
    self.assertEquals("Foo.__hash__", s.name)
    s.teardown()

  def test_teardown(self):
    class Foo(object):pass
    obj = Foo()
    orig = obj.__hash__
    s = StubMethodWrapper( obj.__hash__ )
    s.teardown()
    self.assertEquals( orig, obj.__hash__)

class StubMethodWrapperDescriptionTest(unittest.TestCase):
  
  def test_init(self):
    class Foo(object):pass
    s = StubWrapperDescriptor( Foo, '__hash__' )
    self.assertEquals( s._obj, Foo )
    self.assertEquals( s._attr, '__hash__' )
    self.assertEquals( s, getattr(Foo,'__hash__') )

  def test_name(self):
    class Foo(object):pass

    s = StubWrapperDescriptor(Foo, '__hash__')
    self.assertEquals("Foo.__hash__", s.name)
    s.teardown()

  def test_teardown(self):
    class Foo(object):pass
    orig = Foo.__hash__
    s = StubWrapperDescriptor( Foo, '__hash__' )
    s.teardown()
    self.assertEquals( orig, Foo.__hash__)



'''
Contains sample classes and situations that we want to test.
'''

# Can test module import
from collections import deque

def mod_func_1(*args, **kwargs):
  pass

def mod_func_2(*args, **kwargs):
  mod_func_1(*args, **kwargs)

# For testing when the module has an instance's classmethod bound
class ModInstance(object):
  @classmethod
  def foo(self): pass

mod_instance = ModInstance()
mod_instance_foo = mod_instance.foo

class SampleBase(object):

  a_class_value = 'sample in a jar'

  # Can test __init__
  # Can test variable args
  # Can test changing module import
  def __init__(self, *args, **kwargs):
    self._args = args
    self._kwargs = kwargs
    self._prop_value = 5
    self._deque = deque()

  # Can test a simple property getter
  @property
  def prop(self):
    return self._prop_value

  # Can test property setter
  @prop.setter
  def set_property(self, val):
    self._prop_value = val

  # Can test property deleter
  @prop.deleter
  def del_property(self):
    self._prop_value = None

  @staticmethod
  def a_staticmethod(arg):
    return str(arg)

  # Can test a class method
  @classmethod
  def a_classmethod(cls):
    return cls.a_class_value

  def add_to_list(self, value):
    self._deque.append( value )

  # Can test a bound method
  def bound_method(self, arg1, arg2='two'):
    self._arg1 = arg1
    self._arg2 = arg2

  # Can test that we call another method
  def callback_source(self):
    self.callback_target()
  def callback_target(self):
    self._cb_target = 'called'


class SampleChild(SampleBase):

  # Can test calling super
  # Can test when bound method is overloaded
  def bound_method(self, arg1, arg2='two', arg3='three'):
    super(SampleBase,self).bound_method(arg1, arg2)
    self._arg3 = arg3

  # Can test overloading classmethod
  @classmethod
  def a_classmethod(cls):
    return 'fixed value'

  # Can test overloading a property
  @property
  def prop(self):
    return 'child property'
